/*
    This file is part of the KDE system
    Copyright (C)  1999,2000 Boloni Laszlo

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.

    gofaiConfig.cpp

    configuration items for GOFAI

*/

#include "gofaiConfig.h"
#include <fstream.h>

gofaiConfig* gofaiConfig::_instance = 0;

/**
   Fill in some reasonable results
 */
gofaiConfig::gofaiConfig() {
   // files
   defaultfile = "My.gof";
   currentFileIsRemote = false;
   currentRemoteURL = "";
   autosave = true;
   autosaveInterval= 5 * 60 * 1000;

   // user interface
   toolbarVisible = true;
   statusbarVisible = false;
   detailWindowVisible = true;
   menubarVisible = true;
   remindsMe = true;
   askDelete = true;
   maxNodesInView = 20;
   maxLinksInView = 100;

   fontbase = 15.0;
   fontscale = 1.0;
   speedbase = 15.0;
   speedscale = 1.0;
   animation = true;

   fontname = "helvetica";
   regular_bg = QColor(255,255,255);
   category_bg = QColor(94,181, 243);
   positive_link = QColor(0,0,0);
   negative_link = QColor(255,0,0);
   transparentBg = false;
   dragInitDistance = 20;


   // initializing the ontologies
   current_ontology = new gofaiOntology();
   current_ontology->name = "Common sense";
   ontologies.append(current_ontology);
   // initializing the content types
   current_content_type = new gofaiContentType();
   current_content_type->name = "English";
   current_content_type->subname = "Human readable";
   content_types.append(current_content_type);
   // initializing the link types
   current_link_type = new gofaiLinkType();
   current_link_type->name = "reminds-me";
   link_types.append(current_link_type);

}

/**
   Provides the fontsize
 */
int gofaiConfig::getFontSize(int level) {
  float levelfactor =(3.0) / (level+3.0);
  return (int) (fontbase * fontscale * levelfactor);
}

/**
   Provides the animation speed
 */
int gofaiConfig::getAnimationSpeed() {
  //  return (int) (speedbase * speedscale);
  return (int) speedbase;
}


/**
   Sets the font scale based on the size of the widget
 */
void gofaiConfig::setScale(int h, int w) {
  float defw=400.0;
  float m;
  if (h > w) {
    m=w;
  } else {
    m=h;
  }
  fontscale = m / defw;
  speedscale = defw / m;
}


/**
   Saving into the config file
 */
void gofaiConfig::saveConfig(KConfig *kconf) {
    kconf->setGroup("Files");
    kconf->writeEntry("filename",defaultfile);
    kconf->writeEntry("currentFileIsRemote", currentFileIsRemote);
    kconf->writeEntry("currentRemoteURL", currentRemoteURL.prettyURL());

    kconf->writeEntry("autosave",autosave);
    kconf->writeEntry("autosaveInterval",autosaveInterval);

    kconf->setGroup("View");

    kconf->writeEntry("fontbase", fontbase);
    kconf->writeEntry("speedbase", speedbase);
    kconf->writeEntry("animation", animation);
    kconf->writeEntry("regular_bg", regular_bg);
    kconf->writeEntry("category_bg", category_bg);
    kconf->writeEntry("positive_link", positive_link);
    kconf->writeEntry("negative_link", negative_link);
    kconf->writeEntry("transparentBg", transparentBg);

    kconf->setGroup("UserInterface");
    kconf->writeEntry("remindsMe", remindsMe);
    kconf->writeEntry("askDelete", askDelete);
    kconf->writeEntry("maxNodesInView",maxNodesInView);
    kconf->writeEntry("maxLinksInView",maxLinksInView);
    kconf->writeEntry("toolbarVisible", toolbarVisible);
    kconf->writeEntry("statusbarVisible", statusbarVisible);
    kconf->writeEntry("detailWindowVisible", detailWindowVisible);

    // write the link types
    kconf->setGroup("Link Types");
    gofaiLinkType *glt;
    int i=0;
    for ( glt=link_types.first(); glt != 0; glt=link_types.next() ) {
        QString config_name;
        config_name.sprintf("Name%d", i);
        kconf->writeEntry(config_name, glt->name);
        config_name.sprintf("SubName%d", i);
        kconf->writeEntry(config_name, glt->subname);
        config_name.sprintf("Bidirectional%d", i);
        kconf->writeEntry(config_name, glt->bidirectional);
        i++;
    }
    kconf->writeEntry("LinkTypes", i);

}

/**
   Loading from the config file
 */
void gofaiConfig::loadConfig(KConfig *kconf) {
    QString temp;
    kconf->reparseConfiguration();
    kconf->setGroup("Files");
    defaultfile = kconf->readEntry("filename",defaultfile);
    if (!temp.isEmpty()) {
        defaultfile = temp;
    }

    currentFileIsRemote = kconf->readBoolEntry("currentFileIsRemote",
                                               currentFileIsRemote);
    currentRemoteURL = kconf->readEntry("currentRemoteURL",
                                        "");
    autosave = kconf->readBoolEntry("autosave",autosave);
    autosaveInterval = kconf->readNumEntry("autosaveInterval", autosaveInterval);

    kconf->setGroup("View");

    fontbase = kconf->readDoubleNumEntry("fontbase", fontbase);
    speedbase = kconf->readDoubleNumEntry("speedbase", speedbase);
    animation = kconf->readBoolEntry("animation", animation);

    regular_bg = kconf->readColorEntry("regular_bg", &regular_bg);
    category_bg = kconf->readColorEntry("category_bg", &category_bg);
    positive_link = kconf->readColorEntry("positive_link", &positive_link);
    negative_link = kconf->readColorEntry("negative_link", &negative_link);
    transparentBg = kconf->readBoolEntry("transparentBg", transparentBg);
    kconf->setGroup("UserInterface");
    maxNodesInView = kconf->readNumEntry("maxNodesInView", maxNodesInView);
    maxLinksInView = kconf->readNumEntry("maxLinksInView", maxLinksInView);
    askDelete = kconf->readBoolEntry("askDelete", askDelete);
    remindsMe = kconf->readBoolEntry("remindsMe", remindsMe);
    toolbarVisible = kconf->readBoolEntry("toolbarVisible", toolbarVisible);
    statusbarVisible = kconf->readBoolEntry("statusbarVisible",
                                            statusbarVisible);
    detailWindowVisible = kconf->readBoolEntry("detailWindowVisible",
                                            detailWindowVisible);
     // load the link types
    kconf->setGroup("Link Types");
    int linktypes_no =  kconf->readNumEntry("LinkTypes",0);
    gofaiLinkType *glt;
    if (linktypes_no != 0) {
        // remove the default one
        link_types.setAutoDelete(true);
        while (link_types.removeFirst());
        link_types.setAutoDelete(false);
    }
    for (int i=0; i!=linktypes_no; i++) {
        glt = new gofaiLinkType();
        QString config_name;
        config_name.sprintf("Name%d", i);
        glt->name = kconf->readEntry(config_name,"");
        config_name.sprintf("SubName%d", i);
        glt->subname = kconf->readEntry(config_name, "");
        config_name.sprintf("Bidirectional%d", i);
        glt->bidirectional = kconf->readBoolEntry(config_name, true);
        link_types.append(glt);
    }

}

gofaiConfig * gofaiConfig::getInstance() {
    if (_instance == 0) {
        _instance = new gofaiConfig();
    }
    return _instance;
}
