/* 
    This file is part of the KDE system
    Copyright (C)  1999 Boloni Laszlo
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.

    gofaiVisualArrangement.cpp
 
    Arranges the view in a nice visual interface.
 
*/ 
#include <math.h>
#include <stdlib.h>
#include <fstream.h>
#include "gofaiDataView.h"
#include "gofaiVisualArrangement.h"

/**
   gofaiPosition
 */

gofaiPosition::gofaiPosition() {
    it = NULL;
    occupied = false;
}

void gofaiPosition::setFinalPositions() {
    if (occupied) {
	it->moveTo(QPoint(centerx + rx * sin(angle), 
			  centery + ry * cos(angle)));
    }
}

void gofaiPosition::Randomize(float rnd_percent) {
    float rn;
    rn = (rnd_percent * rand()) / ((float)RAND_MAX);
    rx  += (int) (rx * rn);
    rn = (rnd_percent * rand()) / ((float)RAND_MAX);
    ry  += (int)(ry * rn);
    rn = (rnd_percent * rand()) / ((float)RAND_MAX);
    angle += angle * rn;
}

float gofaiPosition::
      totalLengthOfConnections(gofaiPosition *pos, 
			       gofaiVisualItem *it2, 
			       gofaiDataView *dv) {
    float value = 0.0;
    if (!occupied) return 0.0;
    // now look for a link
    for (int i=0; i!=dv->count; i++) {
	gofaiItem *link = dv->vds[i]->val;
	if ( (link->type == Link) &&
	     ((link->to == it->val->id && link->from == it2->val->id) ||
	      (link->to == it2->val->id && link->from == it->val->id)))
	     {
		 float x = centerx + rx * sin(angle);
		 float y = centery + ry * sin(angle);
		 float x2 = pos->centerx + pos->rx * sin(pos->angle);
		 float y2 = pos->centery + pos->ry * sin(pos->angle);
		 value = sqrt( (x-x2)*(x-x2) + (y-y2)*(y-y2));
		 break;
	     }
    }
    return value;
}
/**
   gofaiCircle
 */

gofaiCircle::gofaiCircle() {
    elements = 0;
    angle_seed = 0.0;
}

void gofaiCircle::setUnoccupied() {
    for(int i=0; i!=elements; i++) {
	pos[i].occupied = false;
    }
}

void gofaiCircle::setFinalPositions() {
    for(int i=0; i!=elements; i++) {
	pos[i].setFinalPositions();
    }
}

void gofaiCircle::Randomize(float rnd_percent) {
    for(int i=0; i!=elements; i++) {
	pos[i].Randomize(rnd_percent);
    }
}

void gofaiCircle::computePositions(int centerx, int centery, int rx, int ry) {
    //    cerr << "ComputePositions " << elements << "\n";
    for(int i=0; i!=elements; i++) {
	pos[i].rx = rx;
	pos[i].ry = ry;
	pos[i].centerx = centerx;
	pos[i].centery = centery;	
	pos[i].angle = angle_seed + (2 * i * M_PI) / elements;
    }    
};

float gofaiCircle::
      totalLengthOfConnections(gofaiPosition *pos, 
			       gofaiVisualItem *it, 
			       gofaiDataView *dv) {
   float value = 0.0;
   for(int i=0; i!=elements; i++) {
	value += pos[i].totalLengthOfConnections(pos, it, dv);
   }
   return value;
}


/**
   gofaiConcentricArrangement
 */

gofaiConcentricArrangement::gofaiConcentricArrangement() {
    circles = 0;
}

void gofaiConcentricArrangement::setFinalPositions() {
    for(int i=0; i!=circles; i++) {
	cir[i].setFinalPositions();
    }
}

void gofaiConcentricArrangement::Randomize(float rnd_percent) {
    for(int i=0; i!=circles; i++) {
	cir[i].Randomize(rnd_percent);
    }
}

void gofaiConcentricArrangement::computePositions(int centerx, int centery, 
						  int rx, int ry) {
    if (circles == 1) {
	cir[0].computePositions(centerx, centery, 0, 0);
	return;
    }
    for(int i=0; i!=circles; i++) {
	cir[i].computePositions(centerx, centery, 
				(rx * i) / (circles-1),
				(ry * i) / (circles-1));
    }
}

/* for the time being */
float gofaiConcentricArrangement::
      totalLengthOfConnections(gofaiPosition *pos, 
			       gofaiVisualItem *it, 
			       gofaiDataView *dv) {
   float value = 0.0;
   for(int i=0; i!=circles; i++) {
	value += cir[i].totalLengthOfConnections(pos, it, dv);
   }
   return value;
}

int gofaiConcentricArrangement::arrange(gofaiDataView &dv,
					int centerx, int centery, 
					int rx, int ry) {
    // decide on the number of elements
    for (int i = 0; i!=dv.count; i++) {
	if (dv.vds[i]->val->type == Node) {
	    if (circles <= dv.vds[i]->level) {
		circles = dv.vds[i]->level+1;
	    }
	    cir[dv.vds[i]->level].elements++;
	    /*
	    cerr << "set elem on level " << dv.vds[i]->level << " to "
		 << cir[dv.vds[i]->level].elements << "\n";
	    */
	}
    }
    
    computePositions(centerx, centery, rx, ry);
    // arrange first those which are not new on their old places, if possible
    for (int i = 0; i!=dv.count; i++) {
	if (dv.vds[i]->val->type == Node && !dv.vds[i]->newentry) {
		// do it later, now these are all new entries
	    cerr << "It should not be any old entry!";
	}
    }
    // arrange the new ones, such that the interconnections are the shortest 
    // possible
    for (int i = 0; i!=dv.count; i++) {
	dv.vds[i]->placed = false;
	dv.vds[i]->link_count = dv.countConnections(dv.vds[i]);
    }
    
    //    for (int lnks = count; lnks>=0; lnks--) 
    for (int lnks = 0; lnks <= dv.count; lnks++) 
	for (int lvl = MAX_LEVELS; lvl >= 0; lvl--)
	//    for (int lvl = 0; lvl != MAX_LEVELS; lvl++)
    for (int i = 0; i!=dv.count; i++) {
	gofaiVisualItem *vit = dv.vds[i];
	if (vit->placed) continue;
	if (vit->link_count  !=  lnks) continue;
	if (vit->level != lvl) continue;
	if (vit->val->type == Node && vit->newentry) {
	    gofaiPosition *best=NULL;
	    float score = 100000;
	    gofaiCircle *circle;
	    circle = &(cir[vit->level]);
	    for(int j=0; j!=circle->elements; j++) {
		gofaiPosition *current = &(circle->pos[j]);
		if (!current->occupied) {
		    float newscore = totalLengthOfConnections(current, 
								 vit,
								 &dv);
		    if (newscore < score) {
			best = current;
		    }
		}
	    }
	    // ok, we found the best position, put it there
	    best->occupied = true;
	    best->it = dv.vds[i];
	}
    }
    Randomize(0.05);
    setFinalPositions();
    return 0; // FIXME: no sense of this here
}

